/**
 * This file is part of the dcrypt project.
 *
 * Copyright: Copyright (C) dcrypt contributors 2008. All rights reserved.
 * License:   MIT
 * Authors:   Thomas Dixon
 */

module dcrypt.crypto.Cipher;

public import dcrypt.crypto.errors.InvalidKeyError;
public import dcrypt.crypto.errors.ShortBufferError;
public import dcrypt.crypto.errors.NotInitializedError;
public import dcrypt.crypto.errors.InvalidParameterError;

public import dcrypt.crypto.params.CipherParameters;

/** Base symmetric cipher class */
abstract class Cipher {
    static const bool ENCRYPT = true,
                      DECRYPT = false;
    
    /**
     * Initialize a cipher.
     * 
     * Params:
     *     encrypt = True if we are encrypting.
     *     params  = Parameters to be passed to the cipher. (Key, rounds, etc.)
     */
    void init(bool encrypt, CipherParameters params);
    
    /**
     * Process a block of plaintext data from the input array
     * and return the encrypted data.
     *
     * Params:
     *     input_  = Array containing input data.
     *
     * Returns: The encrypted data.
     */
    ubyte[] process(void[] input_);
    
    /** Returns: The name of the algorithm of this cipher. */
    char[] name();
    
    /** Reset cipher to its state immediately subsequent the last init. */
    void reset();
}
